// globals
import {
  mapURL,
  EVDataURL,
  width,
  mapWidth,
  plotWidth,
  height,
  margin_y,
  margin_x,
  clip,
} from './globals.js';

// setup
import {
  svgMap,
  svgPlot,
  toolTip,
  gradientColors,
  svgGradient,
  gradient,
  projectionSetUp,
  path,
  xValues,
  yValues,
  rScaleSetUp,
  xAxisSetUp,
  yAxisSetUp,
  scatter,
  mapColorScaleSetUp,
} from './setup.js';

// functions
import {
  getDataExtent,
  normalizeCountyName,
  getCircleColor,
  showtoolTip,
  hidetoolTip,
  idled,
  updateChart,
  getCountyColor,
} from './functions.js';

// render
function render(mapData, countyData) {
  //setup path
  var path = d3.geoPath(projectionSetUp(mapData));

  // x y values
  const xAttr = 'EVCount';
  var x = xValues(countyData, xAttr);

  // x y values
  const yAttr = 'RGDPPerCapita';
  var y = yValues(countyData, yAttr);

  // size circles
  const rAttr = 'Population';
  var rScale = rScaleSetUp(countyData, rAttr);

  // x axis and label
  var xAxis = xAxisSetUp(x);

  // y axis and label
  var yAxis = yAxisSetUp(y);

  // add brushing
  var brush = d3
    .brushX()
    .extent([
      [0, 0],
      [plotWidth, height],
    ])
    .on('end', updateChart);

  // append brush to scatter
  scatter.append('g').attr('class', 'brush').call(brush);

  //
  // plot and mouse effects
  //
  scatter
    //.append('g')
    .selectAll('circle')
    .data(countyData.values())
    .join('circle')
    .attr('id', (d) => d.County)
    .attr('cx', (d) => x(d[xAttr]))
    .attr('cy', (d) => y(d[yAttr]))
    .attr('r', (d) => rScale(d[rAttr]))
    .attr('stroke', 'black')
    .attr('stroke-width', 0.5)
    // fill color of circles
    .attr('fill', function (d) {
      return getCircleColor(d.CountyType);
    })

    .on('mouseover', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('r', (d) => rScale(d[rAttr]))
        .attr('fill', 'yellow');
      showtoolTip(
        event,
        data.County,
        data.Population,
        data.EVCount,
        data.RGDP,
        data.EVCountPerCapita,
      );
    })

    .on('mousemove', function (event, data) {
      showtoolTip(
        event,
        data.County,
        data.Population,
        data.EVCount,
        data.RGDPPerCapita,
        data.EVCountPerCapita,
      );
    })

    .on('mouseleave', function (event, data) {
      d3.select(`circle#${this.id}`)
        .attr('r', (d) => rScale(d[rAttr]))
        .attr('fill', getCircleColor(data.CountyType));
      var countyName = d3.select(this).attr('id');
      d3.select(`path#${countyName}`).attr(
        'fill',
        getCountyColor(countyName),
      );
      hidetoolTip();
    });

  //
  // draw map and add mouse effects
  //
  // map coloring
  const mapColorAttr = 'EVCountPerCapita';
  var mapColorScale = mapColorScaleSetUp(
    countyData,
    mapColorAttr,
  );
  function getCountyColor(name) {
    let entry = countyData.get(name);

    if (!(entry == undefined)) {
      return mapColorScale(entry[mapColorAttr]);
    }

    return 'blue';
  }

  svgMap
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    .attr('id', (d) => d.properties.NAME)
    .attr('d', path)
    .attr('stroke', 'black')
    .attr('stroke-width', 0.5)
    // draw each county
    .attr('fill', function (d) {
      return getCountyColor(d.properties.NAME);
    })

    .on('mouseover', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('r', (d) => rScale(d[rAttr]))
        .attr('fill', 'yellow');
      var countyName = data.properties.NAME;
      var countyPopulation =
        countyData.get(countyName).Population;
      var countyEVCount =
        countyData.get(countyName).EVCount;
      var countyRGDPPerCapita =
        countyData.get(countyName).RGDPPerCapita;
      var countyEVCountPerCapita =
        countyData.get(countyName).EVCountPerCapita;
      showtoolTip(
        event,
        countyName,
        countyPopulation,
        countyEVCount,
        countyRGDPPerCapita,
        countyEVCountPerCapita,
      );
    })

    .on('mousemove', function (event, data) {
      var countyName = data.properties.NAME;
      var countyPopulation =
        countyData.get(countyName).Population;
      var countyEVCount =
        countyData.get(countyName).EVCount;
      var countyRGDPPerCapita =
        countyData.get(countyName).RGDPPerCapita;
      var countyEVCountPerCapita =
        countyData.get(countyName).EVCountPerCapita;
      showtoolTip(
        event,
        countyName,
        countyPopulation,
        countyEVCount,
        countyRGDPPerCapita,
        countyEVCountPerCapita,
      );
    })

    .on('mouseleave', function (event, data) {
      d3.select(`path#${this.id}`).attr(
        'fill',
        getCountyColor(data.properties.NAME),
      );

      var countyName = data.properties.NAME;
      var countyType =
        countyData.get(countyName).CountyType;
      var countyPopulation =
        countyData.get(countyName).Population;
      d3.select(`circle#${this.id}`)
        .attr('r', (d) => rScale(d[rAttr]))
        .attr('fill', getCircleColor(countyType));
      hidetoolTip();
    });

  // idleTimeout set timout to null
  var idleTimeout;

  // idle
  function idled() {
    idleTimeout = null;
  }

  function updateChart(event) {
    var extent = event.selection;

    // If no selection, back to initial coordinate. Otherwise, update X axis domain
    if (!extent) {
      if (!idleTimeout)
        return (idleTimeout = setTimeout(idled, 350)); // This allows to wait a little bit
      x.domain(
        d3.extent(
          d3.map(
            countyData.values().filter((d) => d.County),
            (d) => +d[xAttr],
          ),
        ),
      );
    } else {
      x.domain([x.invert(extent[0]), x.invert(extent[1])]);
      scatter.select('.brush').call(brush.move, null); // This remove the grey brush area as soon as the selection has been done
    }

    xAxis
      .transition()
      .duration(1000)
      .call(d3.axisBottom(x));

    scatter
      .selectAll('circle')
      .transition()
      .duration(1000)
      .attr('cx', function (d) {
        return x(d.EVCount);
      })
      .attr('cy', function (d) {
        return y(d.RGDPPerCapita);
      });
  }
}

// run index
Promise.all([d3.json(mapURL), d3.csv(EVDataURL)]).then(
  ([mapData, csvData]) => {
    var countyData = new Map();
    csvData.forEach((row) => {
      row.County = normalizeCountyName(row.County);
      row.Population = +row.Population;
      countyData.set(row.County, row);
    });

    mapData.features.forEach((f) => {
      f.properties.NAME = normalizeCountyName(
        f.properties.NAME,
      );
    });

    render(mapData, countyData);
  },
);
